searchNodes=[{"doc":"Entrypoint module for Ecto related function with Loupe. Ideally, this module should be completely decoupled from any Repo logic and leave that to the app's Repo","ref":"Loupe.Ecto.html","title":"Loupe.Ecto","type":"module"},{"doc":"Same as build_query/2 but with context or with implementation with no assigns","ref":"Loupe.Ecto.html#build_query/2","title":"Loupe.Ecto.build_query/2","type":"function"},{"doc":"Builds an Ecto query from either an AST or a string. It requires an implementation of the Loupe.Ecto.Definition behaviour and supports assigns as a third parameter.","ref":"Loupe.Ecto.html#build_query/3","title":"Loupe.Ecto.build_query/3","type":"function"},{"doc":"","ref":"Loupe.Ecto.html#t:build_query_error/0","title":"Loupe.Ecto.build_query_error/0","type":"type"},{"doc":"The context is the structure that goes through the query building process. It includes the user assigns which are passed down below in the ecto definition, but also includes a couple fields to make sure that query info are validated beforehand. Ecto schema validations Some things are validated before executing the query to avoid crashes during the query execution. These validations includs: Validate that the fetched schema exists and a valid schema. Validate that the queried field are valid on the attached schema and allowed. It also extract the query bindings which are the queries associations and validates them. This is meant to allow someone to use posts.comments.user.name in a query and directly join the posts's comments and comments's user to filter its name. Binding naming Ecto requires named bindings to be atom (which makes total sense). To avoid generating atoms at runtimes in the system, any non-reserverd keyword is cast as string when generating the AST. When building the Ecto query, the join statements uses bindings from a predefined set that you can find a the bottom of this file under the @binding_keys module attribute.","ref":"Loupe.Ecto.Context.html","title":"Loupe.Ecto.Context","type":"module"},{"doc":"Instanciates a context with an implementation and some assigns. The assigns will be passed down to the implementation during to query building process to alter the definition dynamically.","ref":"Loupe.Ecto.Context.html#new/2","title":"Loupe.Ecto.Context.new/2","type":"function"},{"doc":"Puts bindings in the context validation that bindings are either valid fields or associations.","ref":"Loupe.Ecto.Context.html#put_bindings/2","title":"Loupe.Ecto.Context.put_bindings/2","type":"function"},{"doc":"Puts root schema in the context validating it's a valid Ecto schema","ref":"Loupe.Ecto.Context.html#put_root_schema/2","title":"Loupe.Ecto.Context.put_root_schema/2","type":"function"},{"doc":"Checks if a given schema field is allowed","ref":"Loupe.Ecto.Context.html#schema_field_allowed?/3","title":"Loupe.Ecto.Context.schema_field_allowed?/3","type":"function"},{"doc":"Gets implementation schema fields","ref":"Loupe.Ecto.Context.html#schema_fields/2","title":"Loupe.Ecto.Context.schema_fields/2","type":"function"},{"doc":"Gets implementation schemas","ref":"Loupe.Ecto.Context.html#schemas/1","title":"Loupe.Ecto.Context.schemas/1","type":"function"},{"doc":"Same as selectable_fields/1 but for the root schema","ref":"Loupe.Ecto.Context.html#selectable_fields/1","title":"Loupe.Ecto.Context.selectable_fields/1","type":"function"},{"doc":"Gets fields that can be selected infering the foreign keys. For instance, if someone allows user belong_to relationship on Post̀ , the user_id is automatically returned and there is no need for it to be allowed.","ref":"Loupe.Ecto.Context.html#selectable_fields/2","title":"Loupe.Ecto.Context.selectable_fields/2","type":"function"},{"doc":"Sorts binding in order to be join in order","ref":"Loupe.Ecto.Context.html#sorted_bindings/1","title":"Loupe.Ecto.Context.sorted_bindings/1","type":"function"},{"doc":"","ref":"Loupe.Ecto.Context.html#t:assigns/0","title":"Loupe.Ecto.Context.assigns/0","type":"type"},{"doc":"","ref":"Loupe.Ecto.Context.html#t:binding_path/0","title":"Loupe.Ecto.Context.binding_path/0","type":"type"},{"doc":"","ref":"Loupe.Ecto.Context.html#t:bindings/0","title":"Loupe.Ecto.Context.bindings/0","type":"type"},{"doc":"","ref":"Loupe.Ecto.Context.html#t:implementation/0","title":"Loupe.Ecto.Context.implementation/0","type":"type"},{"doc":"","ref":"Loupe.Ecto.Context.html#t:schema/0","title":"Loupe.Ecto.Context.schema/0","type":"type"},{"doc":"","ref":"Loupe.Ecto.Context.html#t:schemas/0","title":"Loupe.Ecto.Context.schemas/0","type":"type"},{"doc":"","ref":"Loupe.Ecto.Context.html#t:t/0","title":"Loupe.Ecto.Context.t/0","type":"type"},{"doc":"Behaviour for defining an Ecto definition. This behaviours defines what the query builder can use to build the query. All function receives the context assigns, meaning that you could add a user's role to the assigns and filter down available schemas for this particular role (example, allow Users query only for Admins) Exmaple In the following example, the Ecto definition prevents non-admin to query User and only allow the :name field for the Game schema. Meaning that non-admin cannot query for other fields than :name when querying games. defmodule MyApp.Loupe.EctoDefinition do @behaviour Loupe.Ecto.Definition @impl Loupe.Ecto.Definition def schemas ( %{ role : &quot;admin&quot; } ) , do : %{ &quot;User&quot; =&gt; MyApp.User , &quot;Game&quot; =&gt; MyApp.Game } def schemas ( _ ) , do : %{ &quot;Game&quot; =&gt; MyApp.Game } @impl Loupe.Ecto.Definition def schema_fields ( Game , %{ role : role } ) when role != &quot;admin&quot; , do : [ :name ] def schema_fields ( _ , _ ) , do : :all @impl Loupe.Ecto.Definition def scope_schemas ( Game , %{ role : role } ) when role != &quot;admin&quot; do Game . not_deleted ( ) end def scope_schemas ( schema , _ ) , do : schema end","ref":"Loupe.Ecto.Definition.html","title":"Loupe.Ecto.Definition","type":"behaviour"},{"doc":"Gets available field for a schema","ref":"Loupe.Ecto.Definition.html#c:schema_fields/2","title":"Loupe.Ecto.Definition.schema_fields/2","type":"callback"},{"doc":"Gets available schemas","ref":"Loupe.Ecto.Definition.html#c:schemas/1","title":"Loupe.Ecto.Definition.schemas/1","type":"callback"},{"doc":"Scopes schema query builder","ref":"Loupe.Ecto.Definition.html#c:scope_schema/2","title":"Loupe.Ecto.Definition.scope_schema/2","type":"callback"},{"doc":"Tooling for writings tests","ref":"Loupe.Fixture.html","title":"Loupe.Fixture","type":"module"},{"doc":"","ref":"Loupe.Fixture.html#create_context/1","title":"Loupe.Fixture.create_context/1","type":"function"},{"doc":"","ref":"Loupe.Fixture.html#load_schemas/1","title":"Loupe.Fixture.load_schemas/1","type":"function"},{"doc":"","ref":"Loupe.Fixture.html#sigil_L/2","title":"Loupe.Fixture.sigil_L/2","type":"function"},{"doc":"","ref":"Loupe.Fixture.html#with_root_schema/1","title":"Loupe.Fixture.with_root_schema/1","type":"function"},{"doc":"Loupe's compiler entrypoint.","ref":"Loupe.Language.html","title":"Loupe.Language","type":"module"},{"doc":"Compiles a query to AST","ref":"Loupe.Language.html#compile/1","title":"Loupe.Language.compile/1","type":"function"},{"doc":"","ref":"Loupe.Language.html#t:compile_error/0","title":"Loupe.Language.compile_error/0","type":"type"},{"doc":"Extracted AST structure query. It uses a basic syntax like [ action ] [ quantifier? ] [ schema ] where [ predicates ] The action is any alphanumeric value. It can be used to specify what do you aim to use the query for. It could ̀ ecto for instance that you query Ecto with, or even ets that lookup an ets table with match spec. The quantifier is used to limit the queries result but can be ommited defaulting to 1 . It supports the following: Positive integer; 1 , 2 , 10 etc... Range: 10..20 , it limits the query to 10 records offsetting to the 10th record all : Returns all the record matching The schema needs to be an idenfifier (non-quoted alphanumeric) that matches the definition's schemas/1 function. The predicates are combination of boolean operators and operand for validation. See the module's type for every support operators but it can basically be a syntax like # could match `get` and run query on Ecto. get 5 User where ( name = &quot;John Doe&quot; ) or ( age &gt; 18 ) # count match `protobuf` to query Ecto and generate Protobufs. protobuf all BoardGame where name like &quot;Catan&quot;","ref":"Loupe.Language.Ast.html","title":"Loupe.Language.Ast","type":"module"},{"doc":"Extracts bindings of an AST","ref":"Loupe.Language.Ast.html#bindings/1","title":"Loupe.Language.Ast.bindings/1","type":"function"},{"doc":"","ref":"Loupe.Language.Ast.html#is_boolean_operator/1","title":"Loupe.Language.Ast.is_boolean_operator/1","type":"macro"},{"doc":"","ref":"Loupe.Language.Ast.html#is_literal/1","title":"Loupe.Language.Ast.is_literal/1","type":"macro"},{"doc":"","ref":"Loupe.Language.Ast.html#is_operand/1","title":"Loupe.Language.Ast.is_operand/1","type":"macro"},{"doc":"","ref":"Loupe.Language.Ast.html#is_reserved_keyword/1","title":"Loupe.Language.Ast.is_reserved_keyword/1","type":"macro"},{"doc":"","ref":"Loupe.Language.Ast.html#is_text_operand/1","title":"Loupe.Language.Ast.is_text_operand/1","type":"macro"},{"doc":"Instanciates the AST","ref":"Loupe.Language.Ast.html#new/4","title":"Loupe.Language.Ast.new/4","type":"function"},{"doc":"Alpha identifier","ref":"Loupe.Language.Ast.html#t:alpha_identifier/0","title":"Loupe.Language.Ast.alpha_identifier/0","type":"type"},{"doc":"Composed bidings from nested querying","ref":"Loupe.Language.Ast.html#t:binding/0","title":"Loupe.Language.Ast.binding/0","type":"type"},{"doc":"Valid boolean operators","ref":"Loupe.Language.Ast.html#t:boolean_operator/0","title":"Loupe.Language.Ast.boolean_operator/0","type":"type"},{"doc":"Literial values usable in comparison","ref":"Loupe.Language.Ast.html#t:literal/0","title":"Loupe.Language.Ast.literal/0","type":"type"},{"doc":"Valid comparison operands","ref":"Loupe.Language.Ast.html#t:operand/0","title":"Loupe.Language.Ast.operand/0","type":"type"},{"doc":"Validation composed predicates","ref":"Loupe.Language.Ast.html#t:predicate/0","title":"Loupe.Language.Ast.predicate/0","type":"type"},{"doc":"Query quantifier to limit the query result count","ref":"Loupe.Language.Ast.html#t:quantifier/0","title":"Loupe.Language.Ast.quantifier/0","type":"type"},{"doc":"Range from one value to another","ref":"Loupe.Language.Ast.html#t:range/0","title":"Loupe.Language.Ast.range/0","type":"type"},{"doc":"Reserved keywords","ref":"Loupe.Language.Ast.html#t:reserved_keyword/0","title":"Loupe.Language.Ast.reserved_keyword/0","type":"type"},{"doc":"","ref":"Loupe.Language.Ast.html#t:t/0","title":"Loupe.Language.Ast.t/0","type":"type"},{"doc":"Live view component that renders the entities. It also support nesting preloading of the relations though events.","ref":"Loupe.PhoenixLiveView.LiveComponent.html","title":"Loupe.PhoenixLiveView.LiveComponent","type":"module"},{"doc":"Applies a function on an entity at a given path. The path can contain integer, for list indexes or atom for struct's key.","ref":"Loupe.PhoenixLiveView.LiveComponent.html#map_entity_at/3","title":"Loupe.PhoenixLiveView.LiveComponent.map_entity_at/3","type":"function"},{"doc":"Renders a list of struct or a struct recursively. It accepts the following attributes: result (required): either list(struct()) or struct() . definition (required): Ecto definition module. definition_assigns (optional): Assigns for the Ecto definition. inspect_modules (optional): List of module that should be rendered directly using inspect/1 . renderer (optional): Implementation of Loupe.PhoenixLiveView.Renderer behaviour.","ref":"Loupe.PhoenixLiveView.LiveComponent.html#render/1","title":"Loupe.PhoenixLiveView.LiveComponent.render/1","type":"function"},{"doc":"","ref":"Loupe.PhoenixLiveView.LiveComponent.html#t:tree/0","title":"Loupe.PhoenixLiveView.LiveComponent.tree/0","type":"type"},{"doc":"","ref":"Loupe.PhoenixLiveView.LiveComponent.html#t:update_function/0","title":"Loupe.PhoenixLiveView.LiveComponent.update_function/0","type":"type"},{"doc":"Module that is meant to be implemented to control how the component renders","ref":"Loupe.PhoenixLiveView.Renderer.html","title":"Loupe.PhoenixLiveView.Renderer","type":"behaviour"},{"doc":"","ref":"Loupe.PhoenixLiveView.Renderer.html#c:render_type/2","title":"Loupe.PhoenixLiveView.Renderer.render_type/2","type":"callback"},{"doc":"Determines how a value should be rendered for given renderer module","ref":"Loupe.PhoenixLiveView.Renderer.html#render_type/3","title":"Loupe.PhoenixLiveView.Renderer.render_type/3","type":"function"},{"doc":"","ref":"Loupe.PhoenixLiveView.Renderer.html#c:struct_link/3","title":"Loupe.PhoenixLiveView.Renderer.struct_link/3","type":"callback"},{"doc":"Gets a structure's link for a given renderer module","ref":"Loupe.PhoenixLiveView.Renderer.html#struct_link/4","title":"Loupe.PhoenixLiveView.Renderer.struct_link/4","type":"function"},{"doc":"","ref":"Loupe.PhoenixLiveView.Renderer.html#t:t/0","title":"Loupe.PhoenixLiveView.Renderer.t/0","type":"type"},{"doc":"Mock schemas and Ecto modules for tests","ref":"Loupe.Test.Ecto.html","title":"Loupe.Test.Ecto","type":"module"},{"doc":"Comment schema","ref":"Loupe.Test.Ecto.Comment.html","title":"Loupe.Test.Ecto.Comment","type":"module"},{"doc":"Example Ecto definition for the modules defined above.","ref":"Loupe.Test.Ecto.Definition.html","title":"Loupe.Test.Ecto.Definition","type":"module"},{"doc":"External key schema","ref":"Loupe.Test.Ecto.ExternalKey.html","title":"Loupe.Test.Ecto.ExternalKey","type":"module"},{"doc":"Post schema","ref":"Loupe.Test.Ecto.Post.html","title":"Loupe.Test.Ecto.Post","type":"module"},{"doc":"Mocked repo","ref":"Loupe.Test.Ecto.Repo.html","title":"Loupe.Test.Ecto.Repo","type":"module"},{"doc":"Callback implementation for Ecto.Repo.aggregate/3 .","ref":"Loupe.Test.Ecto.Repo.html#aggregate/3","title":"Loupe.Test.Ecto.Repo.aggregate/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.aggregate/4 .","ref":"Loupe.Test.Ecto.Repo.html#aggregate/4","title":"Loupe.Test.Ecto.Repo.aggregate/4","type":"function"},{"doc":"Callback implementation for Ecto.Repo.all/2 .","ref":"Loupe.Test.Ecto.Repo.html#all/2","title":"Loupe.Test.Ecto.Repo.all/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.checked_out?/0 .","ref":"Loupe.Test.Ecto.Repo.html#checked_out?/0","title":"Loupe.Test.Ecto.Repo.checked_out?/0","type":"function"},{"doc":"Callback implementation for Ecto.Repo.checkout/2 .","ref":"Loupe.Test.Ecto.Repo.html#checkout/2","title":"Loupe.Test.Ecto.Repo.checkout/2","type":"function"},{"doc":"","ref":"Loupe.Test.Ecto.Repo.html#child_spec/1","title":"Loupe.Test.Ecto.Repo.child_spec/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.config/0 .","ref":"Loupe.Test.Ecto.Repo.html#config/0","title":"Loupe.Test.Ecto.Repo.config/0","type":"function"},{"doc":"Callback implementation for Ecto.Repo.default_options/1 .","ref":"Loupe.Test.Ecto.Repo.html#default_options/1","title":"Loupe.Test.Ecto.Repo.default_options/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.delete/2 .","ref":"Loupe.Test.Ecto.Repo.html#delete/2","title":"Loupe.Test.Ecto.Repo.delete/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.delete!/2 .","ref":"Loupe.Test.Ecto.Repo.html#delete!/2","title":"Loupe.Test.Ecto.Repo.delete!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.delete_all/2 .","ref":"Loupe.Test.Ecto.Repo.html#delete_all/2","title":"Loupe.Test.Ecto.Repo.delete_all/2","type":"function"},{"doc":"A convenience function for SQL-based repositories that forces all connections in the pool to disconnect within the given interval. See Ecto.Adapters.SQL.disconnect_all/3 for more information.","ref":"Loupe.Test.Ecto.Repo.html#disconnect_all/2","title":"Loupe.Test.Ecto.Repo.disconnect_all/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.exists?/2 .","ref":"Loupe.Test.Ecto.Repo.html#exists?/2","title":"Loupe.Test.Ecto.Repo.exists?/2","type":"function"},{"doc":"A convenience function for SQL-based repositories that executes an EXPLAIN statement or similar depending on the adapter to obtain statistics for the given query. See Ecto.Adapters.SQL.explain/4 for more information.","ref":"Loupe.Test.Ecto.Repo.html#explain/3","title":"Loupe.Test.Ecto.Repo.explain/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get/3 .","ref":"Loupe.Test.Ecto.Repo.html#get/3","title":"Loupe.Test.Ecto.Repo.get/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get!/3 .","ref":"Loupe.Test.Ecto.Repo.html#get!/3","title":"Loupe.Test.Ecto.Repo.get!/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get_by/3 .","ref":"Loupe.Test.Ecto.Repo.html#get_by/3","title":"Loupe.Test.Ecto.Repo.get_by/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get_by!/3 .","ref":"Loupe.Test.Ecto.Repo.html#get_by!/3","title":"Loupe.Test.Ecto.Repo.get_by!/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.get_dynamic_repo/0 .","ref":"Loupe.Test.Ecto.Repo.html#get_dynamic_repo/0","title":"Loupe.Test.Ecto.Repo.get_dynamic_repo/0","type":"function"},{"doc":"Callback implementation for Ecto.Repo.in_transaction?/0 .","ref":"Loupe.Test.Ecto.Repo.html#in_transaction?/0","title":"Loupe.Test.Ecto.Repo.in_transaction?/0","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert/2 .","ref":"Loupe.Test.Ecto.Repo.html#insert/2","title":"Loupe.Test.Ecto.Repo.insert/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert!/2 .","ref":"Loupe.Test.Ecto.Repo.html#insert!/2","title":"Loupe.Test.Ecto.Repo.insert!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert_all/3 .","ref":"Loupe.Test.Ecto.Repo.html#insert_all/3","title":"Loupe.Test.Ecto.Repo.insert_all/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert_or_update/2 .","ref":"Loupe.Test.Ecto.Repo.html#insert_or_update/2","title":"Loupe.Test.Ecto.Repo.insert_or_update/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.insert_or_update!/2 .","ref":"Loupe.Test.Ecto.Repo.html#insert_or_update!/2","title":"Loupe.Test.Ecto.Repo.insert_or_update!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.load/2 .","ref":"Loupe.Test.Ecto.Repo.html#load/2","title":"Loupe.Test.Ecto.Repo.load/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.one/2 .","ref":"Loupe.Test.Ecto.Repo.html#one/2","title":"Loupe.Test.Ecto.Repo.one/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.one!/2 .","ref":"Loupe.Test.Ecto.Repo.html#one!/2","title":"Loupe.Test.Ecto.Repo.one!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.preload/3 .","ref":"Loupe.Test.Ecto.Repo.html#preload/3","title":"Loupe.Test.Ecto.Repo.preload/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.prepare_query/3 .","ref":"Loupe.Test.Ecto.Repo.html#prepare_query/3","title":"Loupe.Test.Ecto.Repo.prepare_query/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.put_dynamic_repo/1 .","ref":"Loupe.Test.Ecto.Repo.html#put_dynamic_repo/1","title":"Loupe.Test.Ecto.Repo.put_dynamic_repo/1","type":"function"},{"doc":"A convenience function for SQL-based repositories that executes the given query. See Ecto.Adapters.SQL.query/4 for more information.","ref":"Loupe.Test.Ecto.Repo.html#query/3","title":"Loupe.Test.Ecto.Repo.query/3","type":"function"},{"doc":"A convenience function for SQL-based repositories that executes the given query. See Ecto.Adapters.SQL.query!/4 for more information.","ref":"Loupe.Test.Ecto.Repo.html#query!/3","title":"Loupe.Test.Ecto.Repo.query!/3","type":"function"},{"doc":"A convenience function for SQL-based repositories that executes the given multi-result query. See Ecto.Adapters.SQL.query_many/4 for more information.","ref":"Loupe.Test.Ecto.Repo.html#query_many/3","title":"Loupe.Test.Ecto.Repo.query_many/3","type":"function"},{"doc":"A convenience function for SQL-based repositories that executes the given multi-result query. See Ecto.Adapters.SQL.query_many!/4 for more information.","ref":"Loupe.Test.Ecto.Repo.html#query_many!/3","title":"Loupe.Test.Ecto.Repo.query_many!/3","type":"function"},{"doc":"Callback implementation for Ecto.Repo.reload/2 .","ref":"Loupe.Test.Ecto.Repo.html#reload/2","title":"Loupe.Test.Ecto.Repo.reload/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.reload!/2 .","ref":"Loupe.Test.Ecto.Repo.html#reload!/2","title":"Loupe.Test.Ecto.Repo.reload!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.rollback/1 .","ref":"Loupe.Test.Ecto.Repo.html#rollback/1","title":"Loupe.Test.Ecto.Repo.rollback/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.start_link/1 .","ref":"Loupe.Test.Ecto.Repo.html#start_link/1","title":"Loupe.Test.Ecto.Repo.start_link/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.stop/1 .","ref":"Loupe.Test.Ecto.Repo.html#stop/1","title":"Loupe.Test.Ecto.Repo.stop/1","type":"function"},{"doc":"Callback implementation for Ecto.Repo.stream/2 .","ref":"Loupe.Test.Ecto.Repo.html#stream/2","title":"Loupe.Test.Ecto.Repo.stream/2","type":"function"},{"doc":"A convenience function for SQL-based repositories that translates the given query to SQL. See Ecto.Adapters.SQL.to_sql/3 for more information.","ref":"Loupe.Test.Ecto.Repo.html#to_sql/2","title":"Loupe.Test.Ecto.Repo.to_sql/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.transaction/2 .","ref":"Loupe.Test.Ecto.Repo.html#transaction/2","title":"Loupe.Test.Ecto.Repo.transaction/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.update/2 .","ref":"Loupe.Test.Ecto.Repo.html#update/2","title":"Loupe.Test.Ecto.Repo.update/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.update!/2 .","ref":"Loupe.Test.Ecto.Repo.html#update!/2","title":"Loupe.Test.Ecto.Repo.update!/2","type":"function"},{"doc":"Callback implementation for Ecto.Repo.update_all/3 .","ref":"Loupe.Test.Ecto.Repo.html#update_all/3","title":"Loupe.Test.Ecto.Repo.update_all/3","type":"function"},{"doc":"Post schema","ref":"Loupe.Test.Ecto.Role.html","title":"Loupe.Test.Ecto.Role","type":"module"},{"doc":"Post schema","ref":"Loupe.Test.Ecto.User.html","title":"Loupe.Test.Ecto.User","type":"module"},{"doc":"User external key relation","ref":"Loupe.Test.Ecto.UserExternalKey.html","title":"Loupe.Test.Ecto.UserExternalKey","type":"module"},{"doc":"This is the base case module for all tests.","ref":"Loupe.TestCase.html","title":"Loupe.TestCase","type":"module"},{"doc":"","ref":"Loupe.TestCase.html#checkout_repo/1","title":"Loupe.TestCase.checkout_repo/1","type":"function"},{"doc":"","ref":"Loupe.TestCase.html#start_repo/1","title":"Loupe.TestCase.start_repo/1","type":"function"},{"doc":"An error that occured in the lexer's step","ref":"Loupe.Errors.LexerError.html","title":"Loupe.Errors.LexerError","type":"exception"},{"doc":"","ref":"Loupe.Errors.LexerError.html#t:t/0","title":"Loupe.Errors.LexerError.t/0","type":"type"},{"doc":"An error that occured in the parser's step","ref":"Loupe.Errors.ParserError.html","title":"Loupe.Errors.ParserError","type":"exception"},{"doc":"","ref":"Loupe.Errors.ParserError.html#t:t/0","title":"Loupe.Errors.ParserError.t/0","type":"type"},{"doc":"Loupe is query language for Ecto schema inspection in a safe and configurable manner. You can see this example app to understand how it applies with Ecto.","ref":"readme.html","title":"Loupe","type":"extras"},{"doc":"Until Loupe reaches 1.x.x , it's considered experimental. The syntax will change, APIs will change and structure will too. We'll do our best to respect semantic versioning and avoid big breaking changes but they can happen.","ref":"readme.html#important","title":"Loupe - Important","type":"extras"},{"doc":"If available in Hex , the package can be installed by adding loupe to your list of dependencies in mix.exs : def deps do [ { :loupe , &quot;~&gt; 0.1.0&quot; } ] end Documentation can be generated with ExDoc and published on HexDocs . Once published, the docs can be found at https://hexdocs.pm/loupe .","ref":"readme.html#installation","title":"Loupe - Installation","type":"extras"},{"doc":"The basic syntax has the following format get [ quantifier? ] [ schema ] where [ predicates ] quantifier is how many records you want. You can provide a positive integer ( 1 , 2 , 3 ...), a range ( 1..10 , 10..20 , 50..100 ) or all . schema needs to be an alphanumeric indentifier that you registered in the Definition (See Ecto Usage for exmaple). predicates needs to be a combinaison or operators and boolean operators. Cool stuff You can use k and m quantifiers for numbers. Writing get all User where money &gt; 100k translates to get all User where money &gt; 100000 . Operators The are a couple of basic operators like &lt; , &gt; , &lt;= , &gt;= , = , != . But also some textual operators: in is used with lists, like age in [18, 21] like is used with strings and automatically wraps in % . You can also use the keyword :empty as a null checker like age :empty . Textual operators and :empty can be prefixed with not to negate the expression: not like , not in , age not :empty . For boolean, the binding can be provided as is and prefixed by not for false. Example where active or where not enabled . Boolean Operators So far, the syntax supprts and and or and use parenthese to scope the expressions.","ref":"readme.html#syntax","title":"Loupe - Syntax","type":"extras"},{"doc":"Create a Definition module The Definition module is necessary for Loupe to work with your Ecto schema. In this module you define the schemas that are allowed to be queried and the fields that are permitted for querying. All callbacks accepts a last argument called &quot;assigns&quot;. The assigns are provided to you when evaluating the query allowing you to alter the defition. You could, for instance, add a user's role to the assign and use that role to filter out the allowed schemas so that only admins can query Users. defmodule MyApp.Loupe.Definition do @moduledoc &quot;&quot;&quot; Example Ecto definition for the modules defined above. &quot;&quot;&quot; @behaviour Loupe.Ecto.Definition @schemas %{ &quot;Post&quot; =&gt; Post , &quot;User&quot; =&gt; User , &quot;Role&quot; =&gt; Role } @impl Loupe.Ecto.Definition def schemas ( %{ role : &quot;admin&quot; } ) , do : @schemas def schemas ( _ ) , do : Map . take ( @schemas , [ &quot;Post&quot; , &quot;User&quot; ] ) @impl Loupe.Ecto.Definition def schema_fields ( _ , %{ role : &quot;admin&quot; } ) , do : :all def schema_fields ( Post , _ ) , do : { :only , [ :title , :body ] } def schema_fields ( User , _ ) , do : { :only , [ :email , :posts ] } def schema_fields ( _ , _ ) , do : :all @impl Loupe.Ecto.Definition def scope_schema ( schema , _ ) , do : schema end Once you have this definition, you can try some queries { :ok , ast } = Loupe.Language . compile ( ~s|get all User where age &gt; 18| ) { :ok , ecto_query } = Loupe.Ecto . build_query ( ast , MyApp.Loupe.Definition , %{ role : &quot;admin&quot; } ) Repo . all ( ecto_query )","ref":"readme.html#ecto-usage","title":"Loupe - Ecto usage","type":"extras"},{"doc":"Here are some things that I would like Loupe to support: Sorting a query, current ideas involves get all User order asc inserted_at get all User where age &gt; 10 ordered asc inserted_at . Support some more complex fields prefixed by ~ (or whatever syntax, inspired by elixir's sigils) like the examples below get all Product where price = ~99.99$ and have that use the Elixir money lib. get all Item where ratio = ~1/4 Replace SQLite in test for Postgres and replace like for ilike in the Ecto Query builder. It has been done like that to avoid messing around with a postgres container only for test but I think it's better to go that route for maximum featureset. Implement a LiveView UI lib that shows the strucutres as expandable. Being able to click on a User's posts to automatically preload all of its nested Posts. Also have &quot;block&quot; UI module where you can simply create a query from dropdowns in a form for non-power user. Make lexer and parser swappable. Right now, you are stuck with the internal structure that I came up with. The idea would be to allow some to swap the syntax for anything they want. For instance, a french team could implement a french query language to give to their normal user.","ref":"readme.html#todo","title":"Loupe - Todo","type":"extras"},{"doc":"You can see the CONTRIBUTING.md file to know more about the contributing guidelines. Pull requests are welcome!","ref":"readme.html#contributing","title":"Loupe - Contributing","type":"extras"}]